/*------------------------------------------------------------------------------------
   Name: TMS-Osc Cross.mq4
   Copyright 2010, Xaphod, http://forexwhiz.appspot.com
   
   Description: Oscillator line cross indicator for the Trading Made Simple System
                Similar to the TDI RSI Signal Line and Trade Signal Line.
	          
   Change log: 
       2012-01-12   
          - limits line cross notification by region - cross down above x / cross up below y
          - coding of regional limitation as external parameter - rbetancor
          - coding of acurately recognising a line cross between candles - gaheitman
          - idea: Mob
            http://www.stevehopwoodforex.com 
       2011-05-18. Xaphod, v1.04
          - Fix for repeated alerts
       2011-05-17. Xaphod, v1.03
          - Timestamp in email alert corrected
       2011-05-17. Xaphod, v1.02 
          - Alerts for bar 1 only. Prevents multiple alerts when MT4 reloads bars
       2011-05-16. Xaphod, v1.01 
          - No alerts after initialization
       2011-05-16. Xaphod, v1.00 
          - First Release 
-------------------------------------------------------------------------------------*/
// Indicator properties
#property copyright "Copyright  2010, Xaphod"
#property link      "http://forexwhiz.appspot.com"

#property indicator_chart_window
#property indicator_buffers 5
#property indicator_color1 CLR_NONE
#property indicator_width1 1
#property indicator_color2 CLR_NONE
#property indicator_width2 2
#property indicator_color3 CLR_NONE
#property indicator_width3 2
#property indicator_color4 LimeGreen
#property indicator_width4 1
#property indicator_color5 Red
#property indicator_width5 1

//#include <xDebug.mqh>

// Constant definitions
#define INDICATOR_NAME "TMS-Osc Cross"
#define INDICATOR_VERSION "1.04"
#define INDICATOR_DISP_VC "1.04, 2011, Xaphod"
// Signals
#define SIG_NO 0
#define SIG_UP 1
#define SIG_DN 2
// Arrows
#define ARROW_THIN_UP 225
#define ARROW_THIN_DN 226
#define ARROW_THICK_UP 233
#define ARROW_THICK_DN 234
#define ARROW_HOLLOW_UP 241
#define ARROW_HOLLOW_DN 242
#define ARROW_ROUND_UP 221
#define ARROW_ROUND_DN 222
#define ARROW_DIAG_THIN_UP 228
#define ARROW_DIAG_THIN_DN 230
#define ARROW_DIAG_THICK_UP 236
#define ARROW_DIAG_THICK_DN 238
#define ARROW_DIAG_HOLLOW_UP 246
#define ARROW_DIAG_HOLLOW_DN 248
#define ARROW_FRACTAL_UP 217
#define ARROW_FRACTAL_DN 218
#define ARROW_RIBBON_UP 200
#define ARROW_RIBBON_DN 202
#define ARROW_THUMB_UP 67
#define ARROW_THUMB_DN 68
#define ARROW_FINGER_UP 67
#define ARROW_FINGER_DN 68


// Indicator parameters
extern string    Indi.Version= INDICATOR_DISP_VC;
extern string    RSI.Settings="";
extern int       RSI.Period=13;             // RSI Period
extern int       RSI.Price=PRICE_CLOSE;     // RSI Price Type: CLOSE=0, OPEN=1, HIGH=2, LOW=3, MEDIAN=4, TYPICAL=5, WEIGHTED=6
extern string    RSISignal.Settings="";
extern int       RSISignal.Period=2;        // RSI Signal MA-Filter Period
extern int       RSISignal.Mode=MODE_SMA;   // MA-Filter Method: SMA=0, EMA=1, SMMA=2, LWMA=4
extern int       RSISignal.UpTreshold=32;
extern int       RSISignal.DownTreshold=68;
extern string    TradeSignal.Settings="";
extern int       TradeSignal.Period=7;      // Trade Signal MA-Filter Period
extern int       TradeSignal.Mode=MODE_SMA; // MA-Filter Method: SMA=0, EMA=1, SMMA=2, LWMA=4
extern string    Arrow.Settings="";
extern int       Arrow.Type=2;              // 0=Thin, 1=Thick, 2=Hollow, 3=Round, 4=Diagonal Thin, 6=Diagonal Thick, 7=Diagonal Hollow, 8=Fractal, 9=Ribbon, 10=Thumb, 11=Finger
extern int       Arrow.Size=1;              // Size of arrow (1-5)
extern int       Arrow.Spacing=25;          // Offset from end of candle to show arrow
extern string    Alert.Settings="";
extern bool      Alert.Popup=false;         // Popup window & sound on alert
extern bool      Alert.Email=false;         // Send email on alert
extern string    Alert.Subject="Default";   // Email Subject. If "Default" the subject is set by the indicator


// Global module varables
double gadRSI[];
double gadRSISig[];
double gadTradeSig[];
double gadArrowUp[];
double gadArrowDn[];
bool gbInit;


//Calculate the point of intersection between two line segments on adjacent candles - gah
//Line Segment 1: (x1,y1), (x2,y2)
//Line Segment 2: (x3,y3), (x4,y4)
//from http://paulbourke.net/geometry/lineline2d/
//Used general case formula for illustrative purposes, despite assumption that
//   x1 and x3 are at the origin, and x2 and x4 are at 1
double CalcYIntersection(double y1,double y2,double y3,double y4) {

   double ua = ((1 - 0)*(y1 - y3) - (y4 - y3)*(0 - 0)) /
                ((y4 - y3)*(1 - 0) - (1 - 0)*(y2 - y1));
   
   return(y1 + ua*(y2-y1));
}


//-----------------------------------------------------------------------------
// function: init()
// Description: Custom indicator initialization function.
//-----------------------------------------------------------------------------
int init() {
  int iArrowUp;
  int iArrowDn;  
  
  SetIndexStyle(0, DRAW_LINE);
  SetIndexBuffer(0, gadRSI);
  SetIndexLabel(0,NULL);
  SetIndexStyle(1, DRAW_LINE);
  SetIndexBuffer(1, gadRSISig);
  SetIndexLabel(1,"RSI Signal");
  SetIndexStyle(2, DRAW_LINE);
  SetIndexBuffer(2, gadTradeSig);
  SetIndexLabel(2,"Trade Signal");
  SetArrowType(iArrowUp, iArrowDn );
  SetIndexStyle(3,DRAW_ARROW,EMPTY,Arrow.Size);
  SetIndexBuffer(3,gadArrowUp);
  SetIndexLabel(3,"Trend Up");
  SetIndexArrow(3, iArrowUp);
  SetIndexStyle(4,DRAW_ARROW,EMPTY,Arrow.Size);
  SetIndexBuffer(4,gadArrowDn);
  SetIndexLabel(4,"Trend Down");
  SetIndexArrow(4, iArrowDn);
  
  // Clear Buffers
  for (int i=0;i<Bars;i++) {
    gadArrowUp[i]=EMPTY_VALUE;
    gadArrowDn[i]=EMPTY_VALUE;
  } 
  gbInit=True;
  
  IndicatorDigits(1);
  IndicatorShortName(INDICATOR_NAME);
  return(0);
}


//-----------------------------------------------------------------------------
// function: deinit()
// Description: Custom indicator deinitialization function.
//-----------------------------------------------------------------------------
int deinit() {
   return (0);
}


///-----------------------------------------------------------------------------
// function: start()
// Description: Custom indicator iteration function.
//-----------------------------------------------------------------------------
int start() {
  int iNewBars;
  int iCountedBars; 
  int i;
  static datetime tCurBar=0;
  static double dPixelValue=0.000001;
  static double dPriceMin=0;
  static double dPriceMax=0; 
    
  // Get unprocessed ticks
  iCountedBars=IndicatorCounted();
  if(iCountedBars < 0) return (-1); 
  if(iCountedBars>0) iCountedBars--;
  iNewBars=Bars-iCountedBars;
  
  // Recalc spacing 
  if (dPriceMin!=WindowPriceMin() || dPriceMax!=WindowPriceMax()) {
    dPriceMin=WindowPriceMin();
    dPriceMax=WindowPriceMax();
    dPixelValue=(dPriceMax-dPriceMin)/800;
    iNewBars=Bars;
    tCurBar=0;
  }
  
  // Calc TMS Signals
  for(i=iNewBars; i>=0; i--) {
    gadRSI[i] = iRSI(NULL,0,RSI.Period,RSI.Price,i); 
  }  
  for(i=iNewBars; i>=0; i--) {  
    gadRSISig[i]=iMAOnArray(gadRSI,0,RSISignal.Period,0,RSISignal.Mode,i);
    gadTradeSig[i]=iMAOnArray(gadRSI,0,TradeSignal.Period,0,TradeSignal.Mode,i);    
  }
  
  // Check for cross signal
  for(i=iNewBars; i>=0; i--) {  
    // On new bar do the following
    if (tCurBar<Time[i]) {
      tCurBar=Time[i];      
      CrossDrawArrow(i+1,Arrow.Spacing*dPixelValue);
      if (!gbInit && i==0)
        CrossAlert(i+1);
    }
  }
  
  // Clear Init Flag
  if (gbInit)
    gbInit=False;
  return(0);
}
//+------------------------------------------------------------------+


//-----------------------------------------------------------------------------
// function: DoAlert()
// Description: Alert on signal
//-----------------------------------------------------------------------------
void DoAlert(string sAlertMsg) {
  // Do Alerts        
  sAlertMsg=Symbol()+", "+TF2Str(Period())+": "+sAlertMsg;
  if (Alert.Popup) Alert(sAlertMsg, ", ", INDICATOR_NAME," v",INDICATOR_VERSION);
  if (Alert.Email) {
    if (Alert.Subject=="Default") Alert.Subject=sAlertMsg;
    SendMail( Alert.Subject, "MT4 Alert! "+INDICATOR_NAME+"\n" + TimeToStr(Time[0],TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);          
  }
}


//-----------------------------------------------------------------------------
// function: CrossDrawArrow()
// Description: Draw arrows for signal crosses
//-----------------------------------------------------------------------------
int CrossDrawArrow(int iBar, double dSpacing) { 
  // Clear indicator buffer 
  gadArrowUp[iBar]=EMPTY_VALUE;
  gadArrowDn[iBar]=EMPTY_VALUE;
  
  // Up signal 
  if ((gadRSISig[iBar] > gadTradeSig[iBar]) && (gadRSISig[iBar+1] < gadTradeSig[iBar+1])) {
    if (CalcYIntersection(gadRSISig[iBar+1],gadRSISig[iBar],gadTradeSig[iBar+1],gadTradeSig[iBar]) < RSISignal.UpTreshold) {
       gadArrowUp[iBar]=Low[iBar]-dSpacing;
//       Print(TimeToStr(Time[iBar],TIME_DATE|TIME_MINUTES)," : ",CalcYIntersection(gadRSISig[iBar+1],gadRSISig[iBar],gadTradeSig[iBar+1],gadTradeSig[iBar]));  
    }
  }
  // Down signal
  else if ((gadRSISig[iBar] < gadTradeSig[iBar]) && (gadRSISig[iBar+1] > gadTradeSig[iBar+1])) {
    if (CalcYIntersection(gadRSISig[iBar+1],gadRSISig[iBar],gadTradeSig[iBar+1],gadTradeSig[iBar])>RSISignal.DownTreshold){
       gadArrowDn[iBar]=High[iBar]+dSpacing;  
//       Print(TimeToStr(Time[iBar],TIME_DATE|TIME_MINUTES)," : ",CalcYIntersection(gadRSISig[iBar+1],gadRSISig[iBar],gadTradeSig[iBar+1],gadTradeSig[iBar]));  
    }
  }
  return(0);  
}


//-----------------------------------------------------------------------------
// function: CrossAlert()
// Description: Alert on signal crosses
//-----------------------------------------------------------------------------
void CrossAlert(int iBar) {
  static int iCurSig=SIG_NO;
  // Up signal     
  if (iCurSig!=SIG_UP && (gadRSISig[iBar] > gadTradeSig[iBar]) && (gadRSISig[iBar+1] < gadTradeSig[iBar+1])) {
    if (CalcYIntersection(gadRSISig[iBar+1],gadRSISig[iBar],gadTradeSig[iBar+1],gadTradeSig[iBar]) < RSISignal.UpTreshold) {
      iCurSig=SIG_UP;
      DoAlert("LONG Signal!");
    }
  }
  // Down signal
  else if (iCurSig!=SIG_DN && (gadRSISig[iBar] < gadTradeSig[iBar]) && (gadRSISig[iBar+1] > gadTradeSig[iBar+1])) {
    if (CalcYIntersection(gadRSISig[iBar+1],gadRSISig[iBar],gadTradeSig[iBar+1],gadTradeSig[iBar])>RSISignal.DownTreshold){
      iCurSig=SIG_DN;
      DoAlert("SHORT Signal!");
    }
  }
  return(0);  
}


//-----------------------------------------------------------------------------
// function: TF2Str()
// Description: Convert time-frame to a string
//-----------------------------------------------------------------------------
string TF2Str(int iPeriod) {
  switch(iPeriod) {
    case PERIOD_M1: return("M1");
    case PERIOD_M5: return("M5");
    case PERIOD_M15: return("M15");
    case PERIOD_M30: return("M30");
    case PERIOD_H1: return("H1");
    case PERIOD_H4: return("H4");
    case PERIOD_D1: return("D1");
    case PERIOD_W1: return("W1");
    case PERIOD_MN1: return("MN1");
    default: return("M"+iPeriod);
  }
}


//-----------------------------------------------------------------------------
// function: SetArrowType()
// Description: Set the arrow character
//-----------------------------------------------------------------------------
void SetArrowType(int& iArrowUp, int& iArrowDn) {
  switch (Arrow.Type) {
    case 1:  // Thin
      iArrowDn=ARROW_THIN_DN;
      iArrowUp=ARROW_THIN_UP;
    break;
    case 2:  // Hollow
      iArrowDn=ARROW_HOLLOW_DN;
      iArrowUp=ARROW_HOLLOW_UP;
    break;
    case 3:  // Round
      iArrowDn=ARROW_ROUND_DN;
      iArrowUp=ARROW_ROUND_UP;
    break;    
    case 4: // Thick
      iArrowDn=ARROW_THICK_DN;
      iArrowUp=ARROW_THICK_UP;
    break;
    case 5: // Diagonal Thin
      iArrowDn=ARROW_DIAG_THIN_DN;
      iArrowUp=ARROW_DIAG_THIN_UP;
    break;
    case 6: // Diagonal Thin
      iArrowDn=ARROW_DIAG_THICK_DN;
      iArrowUp=ARROW_DIAG_THICK_UP;
    break;
    case 7: // Diagonal Hollow
      iArrowDn=ARROW_DIAG_THICK_DN;
      iArrowUp=ARROW_DIAG_THICK_UP;
    break;
    case 8: // Fractal
      iArrowDn=ARROW_FRACTAL_DN;
      iArrowUp=ARROW_FRACTAL_UP;
    break;
    case 9: // Ribbon
      iArrowDn=ARROW_RIBBON_DN;
      iArrowUp=ARROW_RIBBON_UP;
    break;
    case 10: // Thumbs
      iArrowDn=ARROW_THUMB_DN;
      iArrowUp=ARROW_THUMB_UP;
    break;
    case 11: // Finger
      iArrowDn=ARROW_FINGER_DN;
      iArrowUp=ARROW_FINGER_UP;
    break;
    default: // Thick
      iArrowDn=ARROW_THICK_DN;
      iArrowUp=ARROW_THICK_UP;
    break;
  }
}


